<?php

namespace Magento\FDPPaymentGateway\Helper;

use \Magento\Framework\App\Helper\AbstractHelper;

class Api extends AbstractHelper
{
	protected $scopeConfig;
	protected $_currencyFactory;
	protected $logger;
	protected $timezone;
	protected $creditmemoFactory;
	
	public function __construct(
		\Magento\Framework\App\Helper\Context $context,
		\Magento\Directory\Model\CurrencyFactory $currencyFactory,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezone,
		\Magento\Sales\Model\Order\CreditmemoFactory $creditmemoFactory
	)
	{
		$this->scopeConfig = $scopeConfig;
		$this->_currencyFactory = $currencyFactory;
		$this->logger = \Magento\Framework\App\ObjectManager::getInstance()->get(\Psr\Log\LoggerInterface::class);
		$this->timezone = $timezone;
		$this->creditmemoFactory = $creditmemoFactory;
		return parent::__construct($context);
	}
	
	public function action($action, $id, $key, $arg1 = null) {
		if(md5($id . $this->scopeConfig->getValue('payment/fdp_gateway/key')) != $key) return array('error' => 'Błąd kontroli');
		switch ($action) {
			case 'R': 
				return $this->reload($id);
				break;
			case 'B':
				return $this->zwrot($id, 1 * $arg1);
				break;
			case 'A':
				return $this->anuluj($id);
				break;
			case 'Z':
				return $this->zatwierdzenie($id);
				break;
			default:
				return array('error' => 'Błąd wewnętrzny');
		}
    }

    function reload($id) {
    	$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    	$order = $objectManager->create('Magento\Sales\Model\Order')->load($id);
    	$payment = $order->getPayment();
    	$status_o = $payment->getData('fdp_status');

    	$tmp = explode('.', $this->scopeConfig->getValue('payment/fdp_gateway/login'));
    	$mid = $tmp[0];
    	$d = $this->timezone->date(new \DateTime($order->getCreatedAt()));
    	$api_resp = $this->callRest('merchant/' . $mid . '/payment/' . $id . ';date=' . $d->format('Y-m-d') . '?format=json');
    	if(!is_array($api_resp)) return array('error' => $api_resp);
    	$resp = $api_resp[count($api_resp) - 1];
    	if($resp->orderCode != $id) return array('error' => 'Błąd id');
    	$status_n = $resp->paymentStatus;
    	
    	if($status_n != $status_o) {
    		$data = array(
    			'amount'			=> $resp->txnAmount->amount / 100,
    			'transaction_id'	=> null
    		);
    		Request::orderStatus($order, $status_n, $data);
    		$order->save();
    		return array('reload' => 1, 'info' => 'Status zaktualizowany, nastąpi odświeżenie strony');
    	}
    	return array('info' => 'Status nie uległ zmianie');
    }

    function zwrot($id, $kwota = null) {
    	$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    	$order = $objectManager->create('Magento\Sales\Model\Order')->load($id);
    	$payment = $order->getPayment();
    	
    	if($payment->getMethod() != 'fdp_gateway') return array('error' => 'Brak płatności FDP');
    	if($kwota == null) $kwota = $order->getBaseTotalPaid() - $order->getBaseTotalRefunded();
    	
    	$ret = $this->apiRefund($id, $kwota);
    	if(!isset($ret['error'])) {
    		$this->_log('zwrot resp=' .print_r($ret, true), true);

    		$amount = $kwota;
    		$baseGrandTotal = $order->getBaseGrandTotal();
    		$amountRefundLeft = $baseGrandTotal - $order->getBaseTotalRefunded();
    		if ($amountRefundLeft < $amount) {
    			$amount = $amountRefundLeft;
    		}
    		$creditmemo = $this->creditmemoFactory->createByOrder($order, array('adjustment_negative' => $baseGrandTotal - $amount));
    		$totalRefunded = $order->getBaseTotalRefunded() + $creditmemo->getBaseGrandTotal();
    		$payment->setShouldCloseParentTransaction($order->getBaseGrandTotal() <= $totalRefunded);
    		$creditmemo->setPaymentRefundDisallowed(true);
    		$creditmemo->setAutomaticallyCreated(true);
    		$creditmemo->addComment(__('The credit memo has been created automatically.'));
    		$creditmemo->save();
    		
    		$was = $payment->getDataUsingMethod('amount_refunded');
    		$payment->setDataUsingMethod('amount_refunded', $was + $creditmemo->getGrandTotal());

    		$payment->setCreatedCreditmemo($creditmemo);
    		$transaction = $payment->addTransaction(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_REFUND, $creditmemo);
    		
    		$baseOrderRefund = $order->getBaseTotalRefunded() + $creditmemo->getBaseGrandTotal();
    		$orderRefund = $order->getTotalRefunded() + $creditmemo->getGrandTotal();
    		$order->setBaseTotalRefunded($baseOrderRefund);
    		$order->setTotalRefunded($orderRefund);
    		
    		$order->addStatusHistoryComment('FDP ' . ($ret['type'] == 1 ? 'zwrot' : 'reversal') . ': ' . $ret['amount']);
    		
    		if(($ret['type'] == 2) && isset($ret['full'])) {
    			$payment->setData('fdp_status', 99);
    			$opis = isset(Request::$statusy[99]) ? Request::$statusy[99] : '???';
    			$order->addStatusHistoryComment('FDP status: [' . 99 . '] ' . $opis . '');
    		}
    		$order->save();
    	}
    	
		return $ret;
    }

    function apiRefund($id, $kwota) {
    	$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    	$order = $objectManager->create('Magento\Sales\Model\Order')->load($id);

    	$nkwota = floor($kwota * 100);
    	$tmp = explode('.', $this->scopeConfig->getValue('payment/fdp_gateway/login'));
    	$mid = $tmp[0];
    	$d = $this->timezone->date(new \DateTime($order->getCreatedAt()));
    	$api_resp = $this->callRest('merchant/' . $mid . '/payment/' . $id . ';date=' . $d->format('Y-m-d') . '?format=json');
    	if(!is_array($api_resp)) return array('error' => $api_resp);
    	$resp1 = $api_resp[count($api_resp) - 1];
    	$this->_log('api resp=' .print_r($resp1, true), true);
		$stat1 = $resp1->paymentStatus;
		if(!in_array($stat1, array(20, 30, 35, 50))) return array('error' => 'Status transakcji nie pozwala na zwrot. Odśwież status.');
		$skwota = 1 * $resp1->txnAmount->amount;
		if($nkwota > $skwota) return array('error' => 'Kwota za wysoka');
		$dcc = ($resp1->transactionMode[0] == 'DCC') && ($resp1->paymentMethod == 'CARD');
		if($dcc) {
			$skwota = 1 * $resp1->dccAmount->amount;
			$cc = $resp1->dccAmount->currencyRate->currencyCode;
			if(abs($order->getBaseTotalPaid() - $kwota) < 0.005) $nkwota = $skwota;
			else {
				$resp4 = $this->callWs('checkDccRate', array('currencyCode' => $cc));
				if(is_string($resp4)) return array('error' => $resp4);
				if(isset($resp4->return->currencyCode) && $resp4->return->currencyCode == $cc) $rate = $resp4->return->rate;
				else return array('error' => 'Brak kursu ' . $cc);
				$plnkwota = $nkwota;
				$nkwota = round($kwota / $rate * 100);
			}
		}
		if($dcc) $amount = '' . ($plnkwota / 100) . ' PLN (DCC: ' . ($nkwota / 100) . ' ' . $cc . ')';
		else $amount = '' . ($nkwota / 100) . ' PLN';
		if($nkwota == 0) return array('error' => 'Kwota za niska: ' . $amount);

		try {
    		$resp2 = $this->callWs('refundECompayment', array('merchantCode' => $mid, 'paymentCode' => $resp1->paymentCode, 'amount' => $nkwota));
		} catch (\Exception $e) {
			return array('error' => 'Błąd API: ' . $e->getMessage());
		}
    	$this->_log('refundECompayment resp=' .print_r($resp2, true), true);
    	$stat2 = $resp2->return->status;
    	if($stat2 == '10') return array('error' => 'Odrzucony reversal/zwrot');
    	if($stat2 == '20') return array('error' => 'Brak możliwości wykonania operacji');

    	if($stat2 == '00') {
    		$pc = $resp1->paymentCode;
    		$resp3 = $this->callWs('findEComPayment', array('merchantCode' => $mid, 'paymentCode' => $pc));
    		$this->_log('findEComPayment resp=' .print_r($resp3, true), true);
    		$stat3 = $resp3->return->paymentStatus;
    		if($nkwota == $skwota) {
    			if($stat3 == 99) return array('reload' => 1, 'type' => 2, 'full' => 1, 'amount' => $amount, 'info' => 'Transakcja zakupu została anulowana, nastąpi odświeżenie strony');
    			else return array('error' => 'Błąd całościowego anulowania transakcji zakupu');
    		}
    		else {
    			if(!$dcc) $pkwota = 1 * $resp3->return->txnAmount->amount;
    			else $pkwota = 1 * $resp3->return->dccAmount->amount;
    			$this->_log('test1 s='.$skwota.' n='.$nkwota.' p='.$pkwota.' s1='.$stat1.' s3='.$stat3 , true);
				if($skwota - $nkwota != $pkwota) return array('error' => 'Częściowe anulowanie transakcji zakupu nieudane, kwota niezgodna');
				$this->_log('test2 s='.$skwota.' n='.$nkwota.' p='.$pkwota.' s1='.$stat1.' s3='.$stat3 , true);
				if($stat1 != $stat3) return array('error' => 'Częściowe anulowanie transakcji zakupu nieudane, status niezgodny');
				return array('reload' => 1, 'type' => 2, 'amount' => $amount, 'info' => 'Transakcja zakupu została częsciowo anulowana, nastąpi odświeżenie strony');
    		}
    	}
    	if($stat2 == '05') {
    		$pc = $resp2->return->refundCode;
    		$resp3 = $this->callWs('findEComPayment', array('merchantCode' => $mid, 'paymentCode' => $pc));
    		$this->_log('findEComPayment resp=' .print_r($resp3, true), true);
    		$stat3 = $resp3->return->paymentStatus;
    		if(in_array($stat3, array(20, 30, 35, 50))) return array('reload' => 1, 'type' => 1, 'amount' => $amount, 'info' => 'Transakcja zwrotu wykonana poprawnie, nastąpi odświeżenie strony'); 
    		elseif($stat3 == 21) return array('error' => 'Transakcja zwrotu nieudana');
    		elseif($stat3 == 40) return array('error' => 'Transakcja zwrotu odrzucona');
    		elseif($stat3 == 51) return array('error' => 'Transakcja zwrotu - rozliczenie nieudane');
    		elseif($stat3 == 99) return array('error' => 'Transakcja zwrotu anulowana');
    		
    		return array('error' => 'Nieznany status transakcji zwrotu: ' . $stat3);
    	}
    	
    	return array('error' => 'Nieznany status operacji zwrotu: ' . $stat2);
    }

    function anuluj($id) {
    	return array('error' => 'Nie uruchomione');
    }
    
    function zatwierdzenie($id) {
    	return array('error' => 'Nie uruchomione');
    }
    
    function waluty() {
    	$lista = $this->scopeConfig->getValue('payment/fdp_gateway/wal_dcc_lista');
    	$waluty = array_slice(explode(',', $lista), 0, 5);
    	$rates = array();
    	foreach ($waluty as $w) {
    		$resp = $this->callWs('checkDccRate', array('currencyCode' => $w));
    		if(is_string($resp)) return array('error' => $resp);
    		if(isset($resp->return->currencyCode) && $resp->return->currencyCode == $w) $rates['PLN'][$w] = 1 / $resp->return->rate;
    	}
    	if(count($rates) > 0) $this->_currencyFactory->create()->saveRates($rates);
    	return array('status' => 'OK:' . count($waluty));
    }

    function callRest($req) {
    	$auth = $this->scopeConfig->getValue('payment/fdp_gateway/login') . ":" . $this->scopeConfig->getValue('payment/fdp_gateway/haslo');
    	$url = $this->scopeConfig->getValue('payment/fdp_gateway/rest');
    	if(!$this->str_ends_with($url, '/')) $url .= '/';
    	$url .= $req;
    	$this->_log('rest req=' . $url . ' auth=' . $auth, true);
    	if(!extension_loaded ('curl')) return 'CURL extension not loaded';
    	$curl = curl_init ();
    	curl_setopt ( $curl, CURLOPT_URL, $url );
    	curl_setopt ( $curl, CURLOPT_RETURNTRANSFER, 1 );
    	curl_setopt ( $curl, CURLOPT_FAILONERROR, false );
    	curl_setopt ( $curl, CURLOPT_USERPWD, $auth);
    
    	$result = curl_exec ( $curl );
    	if($result === false) {
    		$stat = curl_error($curl);
    		curl_close ( $curl );
    		return $stat;
    	}
    	$info = curl_getinfo($curl);
    	if($info['http_code'] >= 400) {
    		$stat = null;
    		if($result) {
    			$json = json_decode($result);
    			if($json) $stat = $json->fault;
    		}
    		if($stat) $resp = $stat;
    		elseif($info['http_code'] == 401) $resp = 'Auth error';
    		else $resp = 'HTTP error ' . $info['http_code'];
    		curl_close ( $curl );
    		return $resp;
    	}
    	curl_close ( $curl );
    	return json_decode($result);
    }
    
    function callWs($func, $req) {
    	if(!extension_loaded('soap')) return 'SOAP extension not loaded';
    	$this->_log('ws req=' . print_r($req, true), true);
    	$login = $this->scopeConfig->getValue('payment/fdp_gateway/login');
    	$pass = $this->scopeConfig->getValue('payment/fdp_gateway/haslo');
    	$url = $this->scopeConfig->getValue('payment/fdp_gateway/endpoint');
    	if(strpos($url, '?wsdl') === false) $url .= '?wsdl';
    	$url = preg_replace('/^https:\/\//', "https://{$login}:{$pass}@", $url);
    	$this->_log('ws url=' . $url, true);
    	$options = array(
    		'login' => $login,
    		'password' => $pass,
    		'debug' => 0
    	);

    	$client = new \SoapClient($url, $options);
    	$response = $client->__soapCall($func, array($req));

    	return $response;
    }

    function str_ends_with($haystack, $needle) {
    	return strrpos($haystack, $needle) + strlen($needle) === strlen($haystack);
    }

    private function _log($text, $info = false)
    {
    	if($info) $this->logger->info('FDP ' . $text);
    	else $this->logger->error('FDP ' . $text);
    }
    
}

?>
