<?php

namespace Magento\FDPPaymentGateway\Helper;

use \Magento\Framework\App\Helper\AbstractHelper;

class Request extends AbstractHelper
{
	private $debug = false;
	protected $logger;
	protected $_transportBuilder;

	public static $statusy = array(
		0  => '---',
		20 => 'Transakcja zainicjowana',
		21 => 'Podczas autoryzacji wystąpił błąd techniczny',
		30 => 'Transakcja zautoryzowana (zakończona)',
		35 => 'Zatwierdzona (do rozliczenia)',
		40 => 'Transakcja odrzucona przez system autoryzacyjny',
		50 => 'Przekazana do rozliczenia',	
		51 => 'Błąd rozliczenia (przekazana ponownie)',
		99 => 'Transakcja anulowana (manualnie w GUI VPOS)'
	);

	public function __construct(
		\Magento\Framework\App\Helper\Context $context,
		\Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
	)
	{
		$this->logger = \Magento\Framework\App\ObjectManager::getInstance()->get(\Psr\Log\LoggerInterface::class);
		$this->_transportBuilder = $transportBuilder;
		return parent::__construct($context);
	}

	public function testRedir($sharedKey) {
		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
		$storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
		$store = $storeManager->getStore();
		$root = $store->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB);
		if(!isset($_GET['id'])) return $root;
		if(!isset($_GET['sess'])) return $root;
		if(md5($_GET['id'] . $sharedKey) != $_GET['sess']) return $root;
		
		return '';
	}

	public function request($posId, $sharedKey, $mail = false)
	{
		$this->_log('FDP req start', true);
		if(!$_POST || !isset($_POST['order_id'])) {
			$this->_log('no input');
			return false;
		}
		
		$status = 0;
		$error = false;
		
		if(isset($_POST['err_code'])) {
			$status = 0;
			$error = $_POST['err_code'];
			$data = array(
				'pos_id'			=> (int) $_POST['pos_id'],
				'order_id'			=> (int) $_POST['order_id'],
				'session_id'		=> $_POST['session_id'],
				'amount'			=> (int) $_POST['amount'],
				'message'			=> $_POST['message'],
				'err_code'			=> $_POST['err_code'],
			);
			if(isset($_POST['transaction_id']))
				$data['transaction_id'] = $_POST['transaction_id'];
		}
		else {
			$status = (int) $_POST['response_code'];
			$data = array(
				'pos_id'			=> (int) $_POST['pos_id'],
				'order_id'			=> (int) $_POST['order_id'],
				'session_id'		=> $_POST['session_id'],
				'amount'			=> (int) $_POST['amount'],
				'response_code'		=> (int) $_POST['response_code'],
				'transaction_id'	=> $_POST['transaction_id'],
				'cc_number_hash'	=> isset($_POST['cc_number_hash']) ? $_POST['cc_number_hash'] : null,
				'bin'				=> isset($_POST['bin']) ? $_POST['bin'] : null,
				'card_type'			=> isset($_POST['card_type']) ? $_POST['card_type'] : null,
				'auth_code'			=> isset($_POST['auth_code']) ? $_POST['auth_code'] : null,
			);
		}
		
		# kod kontrolny
		$controlData = $_POST['controlData'];
		$calcControlData = $this->_calcControlData($data, $sharedKey);
		
		if(($controlData != $calcControlData) && !$this->debug) {
			$text = "Wrong controlData in confirmation request.\r\n  order id: ".$data['order_id'];
			if(isset($data['transaction_id']))
				$text .= "\r\n  FD tid: ".$data['transaction_id'];
			$this->_log($text);
					
			return false;
		}

		# Sesja i POS
		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
		$order = $objectManager->create('Magento\Sales\Model\Order')->load($data['order_id']);
		if($order->getId() == 0)  {
			$text = "Invalid order id in confirmation request.\r\n  order id: ".$data['order_id'];
			$this->_log($text);					
			return false;
		}
		
		$sessId = md5($data['order_id'] . $sharedKey);
		if($data['pos_id'] != $posId || $data['session_id'] != $sessId) {
			$state = \Magento\Sales\Model\Order::STATE_HOLDED;
			if($order->getStatus() != $state) {
				$order->setStatus($state);
				$order->save();
			}

			$text = "Unauthorized data.\r\n  order id: ".$order->getIncrementId();
			if(isset($data['transaction_id']))
				$text .= "\r\n  FD tid: ".$data['transaction_id'];
			$this->_log($text);
						
			return false;
		}
		
		# Błąd przetwarzania
		if($error) {
			$state = ($error == '005')
				? \Magento\Sales\Model\Order::STATE_CANCELED
				: \Magento\Sales\Model\Order::STATE_HOLDED;
			if($order->getStatus() != $state) $order->setStatus($state);
			if($error == '005') $order->setState($state);
			$order->save();
			if($mail) $this->sendPaymentFailedEmail($order, $sharedKey);

			$text = "Payment error.\r\n  error code: ".$data['err_code']."\r\n  msg: ".$data['message']."\r\n  order id: ".$order->getIncrementId();
			if(isset($data['transaction_id']))
				$text .= "\r\n  FD tid: ".$data['transaction_id'];
			$this->_log($text);
						
			return false;
		}
		
		if(!self::orderStatus($order, $status, $data) && $mail) $this->sendPaymentFailedEmail($order, $sharedKey);
		$order->save();
		
		$text = "Transaction complete\r\n  order id: ".$order->getIncrementId() . "  FD tid: ". $data['transaction_id'] . "  status: " . $status . ' mail: ' . $mail;
		$this->_log($text, true);
		
		return true;
	}

	public static function orderStatus($order, $status, $data) {
		$ret = true;
		$opis = isset(self::$statusy[$status]) ? self::$statusy[$status] : '???';
		$payment = $order->getPayment();
		$order->addStatusHistoryComment('FDP status: [' . $status . '] ' . $opis . '');
		$payment->setData('fdp_status', $status);
		
		if(in_array($status, array(30, 35, 50, 51))) // Transakcja zautoryzowana (zakończona), Zatwierdzona (do rozliczenia), Przekazana do rozliczenia, Błąd rozliczenia (przekazana ponownie) 
		{
			# Kwota
			if(round(floatval($order->getBaseGrandTotal()) * 100) != $data['amount']) {
				$state = \Magento\Sales\Model\Order::STATE_HOLDED;
				
				$text = "Wrong payment amount.\r\n  order id: ".$order->getIncrementId();
				if(isset($data['transaction_id']))
					$text .= "\r\n  FD tid: ".$data['transaction_id'];
				$this->_log($text);
			}
			else $state = \Magento\Sales\Model\Order::STATE_PROCESSING;
			
			$data['amount'] = $data['amount'] / 100.0;
			
			if($payment->getAmountPaid() == 0) {
				if($order->getStatus() != $state) $order->setStatus($state);
				if($data['transaction_id'] != null) $payment->setTransactionId($data['transaction_id']);
				$payment->setIsTransactionClosed(0);
				$payment->setCurrencyCode('PLN');
				$payment->setPreparedMessage('FDP status ' . $status);
				$payment->registerCaptureNotification($data['amount']);
				$order->save();
			}
		}
		elseif(in_array($status, array(20))) // Transakcja zainicjowana
		{
			$state = \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT;
			if($order->getStatus() != $state) $order->setStatus($state);
		}
		elseif(in_array($status, array(21, 40))) // Podczas autoryzacji wystąpił błąd techniczny, Transakcja odrzucona przez system autoryzacyjny
		{
			$state = \Magento\Sales\Model\Order::STATE_HOLDED;
			if($order->getStatus() != $state) $order->setStatus($state);
			$ret = false;
		}
		return $ret;
	}

	private function _calcControlData($d, $sharedKey)
	{
		$data = array();
		foreach($d as $k => $w) if($w) $data[$k] = $w;
		# build query
		$query = '';
		foreach($data as $key => $value)
			$query .= ($query ? '&' : '') . $key .'='. $value;
	
		# add shared key binary string
		$l = strlen($sharedKey)/2;
		$sharedKeyBin = '';
		for($i=1; $i<=$l; ++$i)
			$sharedKeyBin .= pack('H*', substr($sharedKey, 2*$i-2, 2));
	
		$query .= $sharedKeyBin;
	
		# hash
		return hash('sha256', $query);
	}
	
	private function sendPaymentFailedEmail($order, $key)
	{
		//$this->inlineTranslation->suspend();
		$adr = $order->getBillingAddress();
		$rcpt = array(
			'email' => $order->getCustomerEmail(),
			'name' => substr($adr->getFirstname(), 0, 20) . ' ' . substr($adr->getLastname(), 0, 30)
		);
		$this->_log('Mail for ' . $rcpt['email'], true);

		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
		$storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
		$store = $storeManager->getStore();
		$url = $store->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB);
				
		$transport = $this->_transportBuilder
			->setTemplateIdentifier('fdp_email_redo')
			->setTemplateOptions(
				[
					'area' => \Magento\Framework\App\Area::AREA_FRONTEND,
					'store' => $store->getId()
				]
			)->setTemplateVars(['id' => $order->getId(), 'oid' => $order->getIncrementId(), 'url' => $url, 'sess' => md5($order->getId() . $key)])
			->addTo($rcpt['email'], $rcpt['name'])
			->getTransport();
		$transport->sendMessage();
		//$this->inlineTranslation->resume();
	}
	
	private function _log($text, $info = false)
	{
		if($info) $this->logger->info('FDP ' . $text);
		else $this->logger->error('FDP ' . $text);
	}
	
}

?>
